@@ -2,6 +2,7 @@ from collections import OrderedDict
 
 import requests
 import six
+import tenacity
 from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
 from tqdm import tqdm
 
@@ -42,6 +43,7 @@ def format_request_body(fields):
         return fields
 
 
+@tenacity.retry(stop=tenacity.stop_after_attempt(3))
 def upload(destination, file, progress_bar=False, filename='features_backup.rcdb', ):
     """
 
@@ -89,6 +91,7 @@ def upload(destination, file, progress_bar=False, filename='features_backup.rcdb
         )
 
 
+@tenacity.retry(stop=tenacity.stop_after_attempt(3))
 def download(source, destination=None, progress_bar=False):
     is_file = False
     if destination is None:
