@@ -1,4 +1,5 @@
 # coding=utf-8
+import csv
 import datetime
 import json
 import os
@@ -8,10 +9,11 @@ from multiprocessing.pool import Pool
 
 import click
 import humanize
+import tqdm
 from prettytable import PrettyTable
 
 from rebotics_sdk.advanced import remote_loaders
-from rebotics_sdk.cli.common import shell, configure, roles
+from rebotics_sdk.cli.common import shell, configure, roles, set_token
 from .renderers import format_full_table
 from .utils import ReboticsCLIContext, app_dir, pass_rebotics_context, read_saved_role, process_role
 from ..advanced.flows import FileUploadError, download_vrcdb_file, ReboticsClassificationDatabase
@@ -20,7 +22,7 @@ from ..advanced.packers import ClassificationDatabasePacker, ClassificationDatab
 from ..constants import RCDB
 from ..providers import AdminProvider, RetailerProvider, sleep
 from ..providers.facenet import FacenetProvider
-from ..utils import Timer
+from ..utils import Timer, uri_validator, download_and_return_id, get_filename_from_url
 
 
 @click.group()
@@ -404,7 +406,7 @@ db/
     click.echo('Written to {}'.format(packed))
 
 
-@api.group()
+@api.group(name='rcdb')
 def rcdb_group():
     pass
 
@@ -447,7 +449,7 @@ def backup_and_download_rcdb(ctx, retailer, model, target, concurrency):
     download_vrcdb_file(packer, target, concurrency=concurrency)
 
 
-@rcdb_group.command()
+@rcdb_group.command(name='virtual-download')
 @click.option('-c', '--concurrency', type=int, default=4)
 @click.argument('target', type=click.Path(exists=False, file_okay=False, dir_okay=True), default='.')
 @click.argument('backup_ids', type=click.INT, nargs=-1)
@@ -481,7 +483,7 @@ def download_rcdb(ctx, backup_ids, target, concurrency):
         download_vrcdb_file(packer, download_folder, concurrency=concurrency)
 
 
-@rcdb_group.command()
+@rcdb_group.command('virtual-unpack')
 @click.option('-c', '--concurrency', type=int, default=4)
 @click.argument('backup', type=click.Path(exists=True, file_okay=True, dir_okay=False))
 @click.argument('target', type=click.Path(exists=False, file_okay=False, dir_okay=True), default='.')
@@ -492,6 +494,180 @@ def unpack_rcdb(backup, target, concurrency):
     download_vrcdb_file(packer, target, concurrency=concurrency)
 
 
+@rcdb_group.command(name="previews-fs")
+@click.argument("argument", type=click.STRING)
+@click.option('-o', "--output", type=click.Path(exists=True, dir_okay=True, file_okay=False), default=".")
+@click.option('-c', '--concurrency', default=8, type=click.INT)
+@pass_rebotics_context
+def rcdb_previews_fs(ctx, argument, output, concurrency):
+    """
+    Prepare previews file structure by the id, filepath or file url of the JSON formatted backup
+    Make sure that extension is a valid JSON and extension in the admin is set like that
+    """
+    output = pathlib.Path(output).absolute()
+    output.mkdir(parents=True, exist_ok=True)
+
+    filepath = output / "definition.json"
+
+    if argument.isdigit():
+        backup_id = int(argument)
+        rcdb_data = ctx.provider.rcdb.get(backup_id)
+        if rcdb_data['extension'] != RCDB.JSON:
+            click.echo("extension is not supported")  # but we will try anyway
+            # raise click.ClickException("Extension is not supported")
+
+        # download to a local folder
+        remote_loaders.download(rcdb_data['backup'], filepath)
+
+    elif uri_validator(argument):
+        # argument is a url
+        remote_loaders.download(argument, filepath)
+
+    else:
+        filepath = argument
+
+    try:
+        with open(filepath, 'rb') as fp:
+            definition = json.load(fp)
+    except:
+        raise click.ClickException(f"File is not supported. Please check it at {filepath}")
+
+    # apply a script
+    try:
+        import pandas as pd
+        df = pd.DataFrame(definition)
+        df.to_csv(output / "definition.csv", quoting=csv.QUOTE_NONNUMERIC)
+        del df
+    except:
+        pass
+
+    with Pool(concurrency) as pool:
+        for _ in tqdm.tqdm(
+                pool.imap_unordered(partial(download_and_return_id, output=output), definition),
+                desc=f"Downloading images to {str(output)}",
+                total=len(definition),
+        ):
+            pass
+        pool.close()
+        pool.join()
+
+
+@api.group()
+def stitching():
+    pass
+
+
+@stitching.command(name='setup')
+@click.argument("argument", type=click.STRING)
+@click.option('-o', "--output", type=click.Path(exists=True, dir_okay=True, file_okay=False), default=".")
+@click.option('-c', '--concurrency', default=8, type=click.INT)
+@pass_rebotics_context
+def stitching_setup(ctx, argument, concurrency, output):
+    """
+    argument - id of stitching debug data from admin, or json file
+    """
+    output = pathlib.Path(output).absolute()
+    output.mkdir(parents=True, exist_ok=True)
+
+    filepath = output / "definition.json"
+
+    if argument.isdigit():
+        backup_id = int(argument)
+        callback_data = ctx.provider.get_core_callback_data(backup_id)
+        definition = callback_data['data']
+
+        ctx.verbose_log("Saving definition locally")
+        with open(filepath, 'w') as fp:
+            json.dump(definition, fp)
+
+    elif uri_validator(argument):
+        # argument is a url
+        raise click.ClickException("not supported")
+    else:
+        ctx.verbose_log("Assuming we have a filepath")
+        filepath = argument
+        try:
+            with open(filepath, 'rb') as fp:
+                definition = json.load(fp)
+        except:
+            raise click.ClickException(f"File is not supported. Please check it at {filepath}")
+
+    if 'data' in definition:
+        definition = definition['data']
+
+    if ctx.verbose:
+        ctx.format_result(definition['context'])
+
+    files_to_load = []  # pair of remote_url to local path
+
+    for call_count, call_data in enumerate(definition['calls']):
+        call_folder = output / f"call_{call_count}"
+        call_folder.mkdir(parents=True, exist_ok=True)
+
+        input_data = call_data['input']
+
+        stitching_input = {
+            **input_data,
+            'frame_paths': [],
+            'output_file': str(call_folder),
+            'result_image': str(call_folder / "local_stitching.jpeg"),
+        }
+
+        del stitching_input['frame_urls']
+        del stitching_input['annotated_frames']
+
+        # add stitching result image
+        try:
+            files_to_load.append([
+                call_data['image_url'], call_folder / "server_stitched_image.jpeg"
+            ])
+        except:
+            ctx.verbose_log("No stitching image is available")
+
+        mask_od_path = call_folder / "mask_od.json"
+        stitching_input['mask_od'] = str(mask_od_path)
+        files_to_load.append([
+            input_data['mask_od_url'], mask_od_path
+        ])
+
+        # register input files
+        frames_folder = call_folder / "frames"
+        frames_folder.mkdir(parents=True, exist_ok=True)
+
+        for frame_url in input_data['frame_urls']:
+            frame_path = frames_folder / get_filename_from_url(frame_url)
+            stitching_input['frame_paths'].append(str(frame_path))
+            files_to_load.append([
+                frame_url, frame_path
+            ])
+
+        frames_folder = call_folder / "annotated_frames"
+        frames_folder.mkdir(parents=True, exist_ok=True)
+
+        for frame_url in input_data['annotated_frames']:
+            frame_path = frames_folder / get_filename_from_url(frame_url)
+            files_to_load.append([
+                frame_url, frame_path
+            ])
+
+        ctx.verbose_log(f'Creating a stitching input for run #{call_count} into {call_folder}')
+        with open(str(call_folder / "stitching_input.json"), 'w') as fp:
+            json.dump(stitching_input, fp)
+
+    ctx.verbose_log(f"Registered files to downloading: {files_to_load}")
+
+    with Pool(concurrency) as pool:
+        for _ in tqdm.tqdm(
+            pool.starmap(remote_loaders.download, files_to_load),
+            desc=f"Downloading files to {str(output)}",
+            total=len(files_to_load),
+        ):
+            pass
+        pool.close()
+        pool.join()
+
+
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
+api.add_command(set_token, 'set_token')
