@@ -6,7 +6,7 @@ import click
 import requests
 from tqdm import tqdm
 
-from rebotics_sdk.cli.common import shell, configure, roles
+from rebotics_sdk.cli.common import shell, configure, roles, set_token
 from .utils import ReboticsCLIContext, pass_rebotics_context, app_dir, read_saved_role, process_role
 from ..providers import DatasetProvider, ProviderHTTPClientException
 from ..utils import download_file, mkdir_p
@@ -168,3 +168,4 @@ def download_ref_backup(ctx, retailer, facenet, output):
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
+api.add_command(set_token, 'set_token')
