@@ -14,7 +14,7 @@ from tqdm import tqdm
 
 from rebotics_sdk import utils
 from rebotics_sdk.advanced.reverse_planogram import make_planogram_df
-from .common import shell, roles, configure
+from .common import shell, roles, configure, set_token
 from .renderers import format_processing_action_output, format_full_table
 from .tasks import task_create_processing_action_for_image, upload_preview_task, \
     task_download_processing_action, task_recalculate_processing, task_cancel_processing, task_requeue_processing
@@ -773,7 +773,7 @@ def processing_action_realogram_report(ctx, concurrency, drop_duplicates, target
             'unique_id',
             'on_shelf_position', 'display_shelf', 'shelf',
             'position',
-            'left', 'top', 'right', 'bottom',
+            'x_min', 'y_min', 'x_max', 'y_max',
             'shelf_bay_id',
         ], dtype=str)
 
@@ -914,3 +914,4 @@ def feature_vectors_export(ctx, source_model, result_model, batch_size):
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
+api.add_command(set_token, 'set_token')
